<?php

declare(strict_types=1);

use PHPUnit\Framework\TestCase;

use hood\vk\core;
use hood\vk\robots\robot;
use hood\vk\tests\settings;

use hood\accounts\vk as account;

/**
 * @testdox API
 */
final class apiTest extends TestCase
{
    use settings;

    /**
     * @var core $core Ядро фреймворка
     */
    private static core $core;

    /**
     * @var account $account Аккаунт
     */
    protected static account $account;

    /**
     * @var robot $robot Робот
     */
    private static robot $robot;

    /**
     * @testdox Инициализация аккаунта
     * @beforeClass
     */
    public function testAccountInit(): void
    {
        // Проверка входных данных
        if (empty(self::$project_id)) {
            self::$markTestSkipped('Не найден идентификатор проекта');
        }

        // Инициализация аккаунта
        self::$account = new account(empty(self::$id) ? rand(0, 10) : self::$id, __DIR__ . '../../../accounts');

        // Запись режима SSL-протокола
        self::$account->ssl = self::$ssl ?? true;

        if (empty(self::$key)) {
            // Если не указан ключ

            // Проверка входных данных
            if (empty(self::$login)) {
                self::$markTestSkipped('Не найден входной аккаунта');
            } else if (empty(self::$password)) {
                self::$markTestSkipped('Не найден пароль аккаунта');
            }

            // Деаутентификация (на всякий случай)
            self::$account->deauth();

            // Аутентификация и генерация ключа
            self::$key = self::$account->auth(self::$login, self::$password)->key(self::$project_id);
        }

        $this->assertNotNull(self::$key, 'Ошибка при инициализации ключа аккаунта');
    }

    /**
     * @testdox Деинициализация аккаунта
     * @afterClass
     */
    public static function testAccountDeinit(): void
    {
        // Инициализация аккаунта
        self::$account = new account(empty(self::$id) ? rand(0, 10) : self::$id, __DIR__ . '../../../accounts');

        // Деаутентификация
        self::$account->deauth();
    }

    /**
     * @testdox Инициализация робота
     * @before
     */
    public function testRobotGroupInit(): void
    {
        // Инициализация ядра
        self::$core = core::init();

        // Сохранение количества роботов
        $count = self::$core->robots;

        // Инициализация робота
        self::$robot = self::$core->group(empty(self::$group_id) ? rand(0, 10) : self::$group_id);

        // Проверка
        $this->assertEquals(self::$core->robots, $count + 1, 'Ошибка при инициализации робота');
    }

    /**
     * @testdox Деинициализация робота
     * @after
     */
    public function testRobotGroupDeinit(): void
    {
        // Очистка реестра
        self::$core->delete();

        // Проверка
        $this->assertEmpty(self::$core->get(self::$robot->id), 'Ошибка при деинициализации робота');
    }

    /**
     * @testdox Инициализация (безопасная)
     */
    public function testInit(): void
    {
        // Безопасная инициализация
        self::$robot->key(self::$group_key)->api->init();
    }

    /**
     * @testdox Реинициализация
     */
    public function testReinit(): void
    {
        // Реинициализация
        self::$robot->key(self::$group_key)->api->reinit();
    }

    /**
     * @testdox Запись робота (повторная)
     */
    public function testWriteRobot(): void
    {
        // Проверка выброса исключения
        $this->expectExceptionMessage('Запрещено перезаписывать робота');

        // Повторная запись робота
        self::$robot->key(self::$group_key)->api->robot = 'robot';
    }

    /**
     * @tesetdox Чтение робота
     */
    public function testReadRobot(): void
    {
        // Чтение робота
        $this->assertNotNull(self::$robot->key(self::$group_key)->api->robot);
    }

    /**
     * @testdox Запись настроек
     */
    public function testWriteSettings(): void
    {
        // Проверка выброса исключения
        $this->expectExceptionMessage('Запрещено перезаписывать настройки');

        // Запись настроек
        self::$robot->key(self::$group_key)->api['settings'] = 'settings';
    }

    /**
     * @testdox Чтение элемента настроек
     */
    public function testReadSettingsElement(): void
    {
        // Проверка выброса исключения
        $this->expectExceptionMessage('Не найдено: settings[\'element\']');

        // Чтение элемента настроек
        self::$robot->key(self::$group_key)->api['element'];
    }

    /**
     * @testdox Запись элемента настроек
     */
    public function testWriteSettingsElement(): void
    {
        // Запись элемента настроек
        self::$robot->key(self::$group_key)->api['element'] = 'element';

        // Проверка
        $this->assertNotNull(self::$robot->api['element']);
    }

    /**
     * @testdox Выбор получателя по идентификатору
     */
    public function testchooseDestinationById(): void
    {
        // Выбор получателя по идентификатору
        self::$robot->key(self::$group_key)->api->chooseDestination(12345);

        // Проверка
        $this->assertNotNull(self::$robot->api['peer_id']);
    }

    /**
     * @testdox Выбор получателей по идентификаторам
     */
    public function testchooseDestinationByIds(): void
    {
        // Выбор получателей по идентификаторам
        self::$robot->key(self::$group_key)->api->chooseDestination([12345, 12345]);

        // Проверка
        $this->assertNotNull(self::$robot->api['user_ids']);
    }

    /**
     * @testdox Выбор получателя по домену
     */
    public function testchooseDestinationByDomain(): void
    {
        // Выбор получателя по домену
        self::$robot->key(self::$group_key)->api->chooseDestination('domain');

        // Проверка
        $this->assertNotNull(self::$robot->api['domain']);
    }
}
